@@ -10,12 +10,12 @@
 use strict;
 use warnings;
 
-use Mail::DKIM::Signature;
+use Mail::Address;
 
 package Mail::DKIM::Common;
 use base "Mail::DKIM::MessageParser";
 use Carp;
-our $VERSION = 0.36;
+our $VERSION = 0.39;
 
 sub new
 {
@@ -33,7 +33,7 @@ sub add_header
 		$algorithm->add_header($line);
 	}
 
-	if ($line =~ /^([^:]+)\s*:(.*)/s)
+	if ($line =~ /^([^:]+?)\s*:(.*)/s)
 	{
 		my $field_name = lc $1;
 		my $contents = $2;
