@@ -47,34 +47,39 @@ sub init
 sub PRINT
 {
 	my $self = shift;
-	my $buf = $self->{buf} . join("", @_);
+	my $buf = $self->{buf};
+	$buf .= @_ == 1 ? $_[0] : join("", @_)  if @_;
 
-	while (length $buf)
-	{
-		if ($self->{in_header} && $buf =~ /^\015\012/s)
-		{
-			$buf = substr($buf, 2);
-			$self->finish_header();
-			$self->{in_header} = 0;
-			next;
-		}
-		elsif ($self->{in_header} && $buf =~ /^(.*?\015\012)[^\ \t]/s)
+	if ($self->{in_header}) {
+		while (length $buf)
 		{
+			if (substr($buf,0,2) eq "\015\012")
+			{
+				$buf = substr($buf, 2);
+				$self->finish_header();
+				$self->{in_header} = 0;
+				last;
+			}
+			if ($buf !~ /^(.+?\015\012)[^\ \t]/s)
+			{
+				last;
+			}
 			my $header = $1;
-			$buf = substr($buf, length($header));
 			$self->add_header($header);
-			next;
+			$buf = substr($buf, length($header));
 		}
-		elsif (!$self->{in_header} && $buf =~ /^(.*?\015\012)/s)
+	}
+
+	if (!$self->{in_header}) {
+		my $j = rindex($buf,"\015\012");
+		if ($j >= 0)
 		{
-			my $body_line = $1;
-			$buf = substr($buf, length($body_line));
-			$self->add_body($body_line);
-			next;
+			$self->add_body(substr($buf, 0, $j+2));
+			substr($buf, 0, $j+2) = '';
 		}
-		last;
 	}
 	$self->{buf} = $buf;
+	return 1;
 }
 
 sub CLOSE
@@ -105,21 +110,22 @@ sub CLOSE
 	}
 	$self->{buf} = "";
 	$self->finish_body;
+	return 1;
 }
 
 sub add_header
 {
-	die "not implemented";
+	die "add_header not implemented";
 }
 
 sub finish_header
 {
-	die "not implemented";
+	die "finish_header not implemented";
 }
 
 sub add_body
 {
-	die "not implemented";
+	die "add_body not implemented";
 }
 
 sub finish_body
